(function($) {

    var SharePointListConfig = function() {
    };

    var throbberUrl = "/download/resources/com.atlassian.confluence.extra.sharepoint/ajax-loader.gif";

    var sitesDropDown = null;
    var sitesThrobber = null;
    var sitesErrorDiv = null;
    var sitesRetryLink = null;
    var sites = null;

    SharePointListConfig.prototype.fields = {
        "string" : {
            "site" : function(param, options) {
                // create a basic field for the "site" parameter, based on the 'select' template.
                var paramDiv = AJS.clone("#macro-param-select-template");
                sitesDropDown = $("select", paramDiv);

                // because we're going to asynchronously load the set of available values for this field, the user
                // is not allowed to interact with the input field until the data is retrieved.  Show a throbber to
                // entertain the user while this is going on.
                sitesDropDown.attr("disabled", "disabled");
                paramDiv.append("<img id=\"spcon-site-loader\" src=\"" + AJS.params.contextPath + throbberUrl + "\">"); // throbber; visible by default.
                sitesThrobber = $("#spcon-site-loader", paramDiv);

                // create a placeholder element to display error text if something goes wrong with the AJAX call.
                paramDiv.append("<div class='error' id='spcon-site-error'/>");
                sitesErrorDiv = AJS.$("#spcon-site-error", paramDiv);
                sitesErrorDiv.append("<div id='spcon-site-errortext' class='error'/>");
                sitesErrorDiv.append("<span id='spcon-site-retry' style='color: blue'>Retry</span>"); // HACK: use a span instead of an anchor because the macro browser attaches an event handler to all anchor elements which I haven't been able to unbind yet.
                sitesRetryLink = AJS.$("#spcon-site-retry", sitesErrorDiv);

                // the macro browser adds an event handler to all anchors which causes them to open in a new window...
                // we need to remove this event handling on the retry link.
                sitesRetryLink.click(retryGetSitesASync);
                sitesErrorDiv.css("display", "none");

                // we need to do some trickery when the value is first set, and when the value is subsequently changed,
                // so set this up.
                if (options == null)
                    options = {};
                // TODO: on change, change the site that is passed to the getlists query.
                //options.onchange = selectedSiteChanged;
                options.setValue = setSiteValue;

                // commence the asynchronous operation to retrieve the list of sites, pronto!
                getSitesASync();

                // Add a default "empty" option (which means to use the configured "default" site.
                sitesDropDown.append(AJS.$("<option/>").attr("value", ""));

                return AJS.MacroBrowser.Field(paramDiv, sitesDropDown, options);
            },

            // First attempt at auto-complete using QuickSearchDropDown
            "listname" : function(param, options) {
                var paramDiv = AJS.clone("#macro-param-template");
                var input = $("input", paramDiv);
                paramDiv.append("<div id='dd-list' class='aui-dd-parent autocomplete'></div>");
                var dddiv = $("#dd-list", paramDiv);

                var ddoptions = {
                    addDropdownData: function (matrix) {
                        matrix.push([{
                            name: "Search",
                            className: "search-for",
                            href: "#"
                        }]);
                        return matrix;
                    },
                    dropdownPlacement: function(dropDown) {
                        //AJS.$("#auto-comple").append(dropDown);
                        dddiv.append(dropDown);
                    },
                    ajsDropDownOptions: {
                        selectionHandler: function (e, selected) {
                        if (!selected) {
                            return;
                        }

                        // extract the information about the selected item from the jQuery data cache for the corresponding <span> element.
                        var dataObj = AJS.$.data(selected.find("span")[0], "properties");

                        input.val(dataObj.name);
                        this.hide();
                        e.preventDefault();
                        }
                    }
                };

                input.quicksearch("/plugins/sharepoint/siteLists.action?siteAlias=", null, ddoptions);

                return AJS.MacroBrowser.Field(paramDiv, input, options);
            }
        }
    };

    function setSiteValue(value)
    {
        // before we set the value, let's ensure that the appropriate option actually exists.
        // if this set is invoked by the macro browser as it loads a pre-existing macro, *and* the
        // asynchronous call to retrieve the list of available sites hasn't returned yet, then the option won't
        // be there.  If this is the case, then we'll temporarily add it in.
        var targetOption = AJS.$("#macro-param-site > option[value='" + value + "']");
        if (targetOption.length == 0)
        {
            // it doesn't exist; we need to create it.
            var option = AJS.$("<option/>");
            option.attr("value", value);
            option.attr("title", value);
            option.text(value);
            sitesDropDown.append(option);
        }

        // now set it.
        sitesDropDown.val(value);
    }

    function updateSitesDropDown()
    {
        // what's the currently selected value?
        var currentValue = sitesDropDown.val();

        // clear that motherfucker out.
        sitesDropDown.empty();
        sitesDropDown.append(AJS.$("<option/>").attr("value", ""));
        
        for (var i = 0; i < sites.length; i++)
        {
            var site = sites[i];
            var option = AJS.$("<option/>");
            option.attr("value", site.alias);
            option.attr("title", site.serverUrl);
            option.text(site.alias);
            sitesDropDown.append(option);
        }

        // restore the currently selected value.
        sitesDropDown.val(currentValue);

        // AJAX complete; ensure that the throbber is hidden and the drop-down is enabled.
        //sitesThrobber.remove();
        sitesThrobber.css("display", "none");
        sitesDropDown.removeAttr("disabled");
    }

    function retryGetSitesASync(event)
    {
        event.preventDefault();
        alert("This was prevented: " + event.type);
        
        // hide the error div again.
        sitesErrorDiv.css("display", "none");

        // show the throbber.
        sitesThrobber.css("display", "inline");

        // do it!
        getSitesASync();

        return false;
    }

    function getSitesASync() {
        var requestSettings = {
            async: true,
            url: AJS.params.contextPath + "/plugins/sharepoint/configuredSites.action",
            dataType: 'json',
            timeout: 30000, // 30 seconds.
            error: function(XMLHttpRequest, textStatus, errorThrown) {

                var errorText;
                if (textStatus == "timeout")
                {
                    // Should be internationalised
                    errorText = "The request to retrieve configured sites from Confluence timed out after 30 seconds.";
                }
                else
                {
                    // Should be internationalised
                    errorText = "The request to retrieve configured sites from Confluence failed.";
                }

                // make the error visible.
                AJS.$("#spcon-site-errortext", sitesErrorDiv).text(errorText);
                sitesErrorDiv.css("display", "inline");

                // hide the throbber.
                sitesThrobber.css("display", "none");
            },
            success: function(data){
                if (!data.length)
                {
                    sitesErrorDiv.append("Configured SharePoint sites result was not in the expected format. <a href='#' onclick='getSitesASync()'>Retry</a>");
                    sites = {};
                }
                
                sites = data;

                // was successful! Make sure the error div is hidden.
                AJS.$("#spcon-site-errortext", sitesErrorDiv).empty();
                sitesErrorDiv.css("display", "none");
                
                updateSitesDropDown();
            }
        };
        $.ajax(requestSettings);
    }

    AJS.MacroBrowser.Macros["sp-link"] = new SharePointListConfig();
    delete AJS.MacroBrowser.Macros["sp-link"].beforeParamsSet;

    AJS.MacroBrowser.Macros["sp-list"] = new SharePointListConfig();
})(AJS.$);